using System.Linq;
using Modular.Flair.WebPackage.Blocks;
using NUnit.Framework;
using NUnit.Framework.SyntaxHelpers;
using Rhino.Mocks;
using Spark.Modules;

namespace Modules.RhinoMocks.Tests.FlairTests.BlockTests
{
    [TestFixture]
    public class FileBasedTests : BlockTestBase
    {
        private FileContentsBlock block;
        private IFileDuck mockFileDuck; 

        [SetUp]
        public override void Setup()
        {
            base.Setup();
            kernel.AddComponentInstance<IFileDuck>(mockFileDuck = MockRepository.GenerateMock<IFileDuck>());
            kernel.AddComponent<FileContentsBlock>();
            block = kernel.Resolve<FileContentsBlock>(); 
        }

        [Test]
        public void The_block_should_display_the_file_contents_if_the_file_exists()
        {   
            // Arrange
            mockFileDuck.Expect(x => x.Exists(null))
                .IgnoreArguments()
                .Return(true);
            mockFileDuck.Expect(x => x.ReadAllText(null))
                .IgnoreArguments()
                .Constraints(Rhino.Mocks.Constraints.Is.NotNull())
                .Return("FooBar");

            // Act
            block.RenderBlock();

            // Assert 
            var seccondParameter = mockHtmlHelper.GetArgumentsForCallsMadeOn(
                x => x.RenderPartial(Arg<string>.Is.Anything, Arg<object>.Is.Anything)).First()[1];

            Assert.That(seccondParameter, Is.EqualTo("FooBar"));

            // Make sure what we expected to happen, did happen
            mockFileDuck.VerifyAllExpectations();            
        }
        
    }
}