using System;
using System.Configuration;
using System.Data.SqlClient;
using System.Web.Mvc;
using Modular.Flair.WebPackage.Blocks;
using Modular.Flair.WebPackage.Models;
using NUnit.Framework;
using Spark.Modules;
using TypeMock;
using TypeMock.ArrangeActAssert;

namespace Modules.TypeMockIsolator.Tests.FlairTests.BlockTests
{
    [TestFixture]
    public class FlairTestsWithTypemock
    {
        [Test]
        public void Mocking_a_static_class_with_typemock()
        {
            // Arrange 
            Isolate.Fake.StaticMethods(typeof(ConfigurationManager), Members.ReturnNulls); // This is optional, but tells Isolator to return null for any call made that does not have a specification
            Isolate.WhenCalled(() => ConfigurationManager.ConnectionStrings["TriviaDb"] ).WillReturn(new ConnectionStringSettings("Foo", "bar"));
            
            // Act
            var block = new DidYouKnowLegacyBlock();

            // Assert
            Isolate.Verify.WasCalledWithAnyArguments(() => ConfigurationManager.ConnectionStrings["TriviaDb"]);
        }

        [Test,Isolated]
        public void Mocking_hard_database_dependencies()
        {
            // Arrange 
            var mockConnection = Isolate.Fake.Instance<SqlConnection>(Members.ReturnRecursiveFakes);
            var mockCommand = Isolate.Fake.Instance<SqlCommand>(Members.ReturnRecursiveFakes);
            var mockReader = Isolate.Fake.Instance<SqlDataReader>(Members.ReturnNulls);
            var mockHtmlHelper = Isolate.Fake.Instance<IHtmlHelperDuck>();
            var mockViewData = Isolate.Fake.Instance<ViewDataDictionary>();
            
            Isolate.Swap.NextInstance<SqlConnection>().With(mockConnection);
            Isolate.Swap.NextInstance<SqlCommand>().With(mockCommand);

            Isolate.WhenCalled(() => ConfigurationManager.ConnectionStrings["TriviaDb"]).WillReturn(new ConnectionStringSettings("Foo", "Bar"));
            Isolate.WhenCalled(() => mockCommand.ExecuteReader()).WillReturn(mockReader);
            Isolate.WhenCalled(() => mockReader["Details"]).WillReturn("Foo");
            Isolate.WhenCalled(() => mockReader["Id"]).WillReturn(42);
            Isolate.WhenCalled(() => mockReader["Teaser"]).WillReturn("FooBar");
            

            // Act
            var block = new DidYouKnowLegacyBlock
                            {
                                Html = mockHtmlHelper, 
                                ViewData = mockViewData
                            };
            block.RenderBlock();

           
            // Assert
            // Cannot get arguments in current version of Isolator with AAA syntax. Old syntax will work though. 
            // With the following: 
            //using (RecordExpectations rec = new RecordExpectations())
            //{
            //    rec.CheckArguments();
            //}
        }

    }
}